# -*- coding: utf-8 -*-
from django.db import models
from django.conf import settings
from django.template.defaultfilters import slugify

import feedparser
import time
import datetime
import socket
import urllib2

from teams.models import Team
from userprofiles.models import UserProfile


def create_feed(user, url):
    data = feedparser.parse(url)
    feed = Feed(
        user=user,
        url=url,
        title=data.feed.title
    )
    feed.save()
    feed.update(data)


def get_entry_id(entry, feed):
    if hasattr(entry, 'id'):
        return entry.id

    if hasattr(entry, 'link'):
        return entry.link

    return '%s:%s' % (feed.id, hash(entry.title))


class Feed(models.Model):
    user = models.ForeignKey(UserProfile, blank=True, null=True)
    team = models.ForeignKey(Team, blank=True, null=True)
    url = models.URLField(verify_exists=False)
    title = models.CharField(max_length=128, null=True, blank=True)
    last_updated = models.DateTimeField(null=False, blank=False, default=datetime.datetime(1970, 1, 1))
    active = models.BooleanField(default=True)

    class Meta:
        ordering = ('title',)

    def __unicode__(self):
        return self.title

    def update(self, data=None, first_load=False):
        if not data:
            old_timeout = socket.getdefaulttimeout()
            socket.setdefaulttimeout(20)
            handlers = []
            proxy = getattr(settings, 'HTTP_PROXY', None)
            if proxy:
                handlers.append(urllib2.ProxyHandler({"http": proxy}))
            data = feedparser.parse(self.url, modified=self.last_updated.utctimetuple(), handlers=handlers)
            socket.setdefaulttimeout(old_timeout)

        if not data or not hasattr(data, 'entries') or len(data.entries) == 0:
            # No new entries for this feed
            return
        now = datetime.datetime.now()
        existing_articles = set(list(Article.objects.filter(source=self).values_list('uid', flat=True)))
        returned_articles = set([])

        if self.title is None or self.title == '':
            self.title = data.feed.title
        self.last_updated = datetime.datetime.fromtimestamp(time.mktime(getattr(data, 'modified', time.gmtime())))
        self.save()

        for entry in data.entries:
            entry_id = get_entry_id(entry, self)
            if Article.objects.filter(source=self, uid=entry_id).count() > 0:
                returned_articles.add(entry_id)
            else:
                pub_date = datetime.datetime.fromtimestamp(time.mktime(entry.get('date_parsed', time.gmtime())))
                # On first load, we want to mark the import as the published date
                # otherwise they all go to the top of the list
                if first_load:
                    imp_date = min(now, pub_date)
                else:
                    imp_date = now

                if hasattr(entry, 'content'):
                    content = entry.content[0].value
                else:
                    content = entry.get('description', '')

                article = Article(
                    source=self,
                    uid=entry_id,
                    author=entry.get('author', '')[:128],
                    published=pub_date,
                    imported=imp_date,
                    title=entry.get('title', '')[:128],
                    link=entry.get('link', None),
                    content=content,
                )
                article.save()

        obsolete_articles = existing_articles - returned_articles
        Article.objects.filter(source=self, uid__in=obsolete_articles).delete()


class Article(models.Model):
    source = models.ForeignKey(Feed, null=True, blank=True)
    uid = models.CharField(max_length=256)
    author = models.CharField(max_length=128, null=True, blank=True)
    published = models.DateTimeField(null=True, blank=True)
    imported = models.DateTimeField(null=True, blank=True)
    link = models.URLField(max_length=1024, null=True, blank=True)
    title = models.CharField(max_length=128, null=True, blank=True)
    content = models.TextField(null=True, blank=True)

    class Meta:
        ordering = ['-imported', '-published', ]

    def __unicode__(self):
        return u'%s (%s)' % (self.title, self.author)

    @models.permalink
    def get_absolute_url(self):
        return ('article-detail', [str(self.id), slugify(self.title)])
