#!/usr/bin/python
# -*- coding: utf-8 -*-
from django.core.management.base import NoArgsCommand
from django.contrib.auth.models import User, Group
from django.db.models.fields import DateTimeField, DateField

from meetings.models import BaseMeeting, TeamMeeting, AgendaItem
from events.models import BaseEvent, GlobalEvent, TeamEvent, TeamEventComment, Attendee
from teams.models import Language, Continent, Country, Team
from venues.models import Venue
from userprofiles.models import UserProfile

import urllib2
import json
import re
import os
import sys
import subprocess
import time

ORDER = [('languages', Language),
         ('continents', Continent),
         ('countries', Country),
         ('venues', Venue),
         ('global', GlobalEvent),
         ('groups', Group),
         ('users', User),
         ('profiles', UserProfile),
         ('teams', Team),
         ('events', TeamEvent),
         ('comments', TeamEventComment),
         ('attendees', Attendee),
         ('meeting', TeamMeeting),
         ('agenda', AgendaItem),
         ]

date_pattern1 = re.compile('^\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}$')
date_pattern2 = re.compile('^\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}.\d+$')


def get_field_data(model, model_data):
    fields = {}
    for field in model._meta.fields:
        field_value = model_data.get(field.name, None)
        if field_value:
            if isinstance(field, DateTimeField):
                if date_pattern1.match(field_value):
                    fields[field.name] = field_value
                elif date_pattern2.match(field_value):
                    fields[field.name] = field_value.split('.')[0]
            elif isinstance(field, DateField):
                if date_pattern1.match(field_value) or date_pattern2.match(field_value):
                    fields[field.name] = field_value.split(' ')[0]
            else:
                fields[field.name] = field_value
    for field in model._meta.many_to_many:
        fields[field.name] = model_data.get(field.name, [])
    return fields


class Command(NoArgsCommand):
    help = "Copy the live data to the local instance"

    def handle_noargs(self, **options):
        t1 = time.time()
        # Save the super user, so it can be restored later
        super_user = None
        try:
            super_user = User.objects.get(pk=1)
        except:
            pass

        # Delete all old data, so the id's do not conflict when linking
        print "Removing local data from database."

        for service, model in ORDER:
            model.objects.all().delete()

        loaddata_json = []
        services_url = 'http://loco.ubuntu.com/services/'

        for service, model in ORDER:
            # Get the json data from the server
            print "Getting %s." % service
            json_data = urllib2.urlopen("%s%s" % (services_url, service)).read()

            for model_data in json.loads(json_data):
                loaddata_json.append({'pk': model_data['id'],
                                      'model': str(model._meta),
                                      'fields': get_field_data(model, model_data)})

                # GlobalEvent, TeamEvent are a subclass of BaseEvent, so we need to create the BaseEvent also
                if model in [GlobalEvent, TeamEvent]:
                    loaddata_json.append({'pk': model_data['id'],
                                          'model': str(BaseEvent._meta),
                                          'fields': get_field_data(BaseEvent, model_data)})

                # TeamMeeting is a subclass of BaseMeeting, so we need to create the BaseMeeting also
                if model is TeamMeeting:
                    loaddata_json.append({'pk': model_data['id'],
                                          'model': str(BaseMeeting._meta),
                                          'fields': get_field_data(BaseMeeting, model_data)})

        print "Storing data into a json file."
        f = open('livedata.json', 'w')
        f.write(json.dumps(loaddata_json))
        f.close()

        print "Loading data into database."
        p = subprocess.Popen([sys.executable, 'manage.py', 'loaddata', 'livedata.json'],
                             stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True)
        log = p.stdout.read()
        p.communicate()
        print "%s" % log

        os.remove('livedata.json')

        # Restore the super user
        if super_user:
            print"Restoring the super user."
            old_su = super_user.__dict__
            del old_su['id']
            if old_su.__contains__('_state'):
                del old_su['_state']
            new_su, created = User.objects.get_or_create(username=old_su['username'])
            new_su.__dict__.update(old_su)
            new_su.save()

        t2 = time.time()
        print "It took %0fsec to finish" % (t2 - t1)
