# -*- coding: utf-8 -*-
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.template.loader import render_to_string
from django.contrib.auth import logout
from django.conf import settings
from django.utils import translation
from django.utils.translation import check_for_language

try:
    from django_openid_auth.exceptions import (
        MissingPhysicalMultiFactor,
        MissingUsernameViolation,
    )
except ImportError:
    # pylint: disable-msg=C0103
    MissingPhysicalMultiFactor = None
    MissingUsernameViolation = None


def index(request):
    from events.models import GlobalEvent, TeamEvent
    from meetings.models import TeamMeeting
    from articles.models import Article
    team_event_count = TeamEvent.objects.next_events().count()
    team_meeting_count = TeamMeeting.objects.next_meetings().count()
    global_event_list = GlobalEvent.objects.next_events()[:5]
    articles = Article.objects.all()[:5]

    context = {'team_event_count': team_event_count,
               'team_meeting_count': team_meeting_count,
               'global_event_list': global_event_list,
               'articles': articles,
               }

    return render_to_response('index.html', context,
        RequestContext(request))


def about(request):
    context = {}
    return render_to_response('about.html', context,
        RequestContext(request))


def about_loco(request):
    context = {}
    return render_to_response('about_loco.html', context,
        RequestContext(request))


def loco_setup(request):
    context = {}
    return render_to_response('loco_setup.html', context,
        RequestContext(request))


def irc_chat(request):
    context = {}
    return render_to_response('irc_chat.html', context,
        RequestContext(request))


def using_locodir(request):
    context = {}
    return render_to_response('using_locodir.html', context,
        RequestContext(request))


def site_logout(request):
    logout(request)
    return HttpResponseRedirect('/')


def set_language(request, next=None):
    """
    Change the language for a user
    """
    if not next:
        next = request.REQUEST.get('next', None)
    if not next:
        next = request.META.get('HTTP_REFERER', None)
        if not next:
            next = '/'
    response = HttpResponseRedirect(next)
    if request.method == 'GET':
        lang_code = request.GET.get('lang', '')
        if lang_code and check_for_language(lang_code):
            if hasattr(request, 'session'):
                request.session['django_language'] = lang_code
            else:
                response.set_cookie(settings.LANGUAGE_COOKIE_NAME, lang_code)
            translation.activate(lang_code)
    return response


# pylint: disable-msg=C0103
def site_search(request):
    from common.forms import SiteSearchForm
    teams = []
    global_events = []
    team_events = []
    venues = []
    meetings = []
    q = None
    form = SiteSearchForm(data=request.GET)
    if form.is_valid():
        q = form.cleaned_data.get('q', None)
        if q is not None:
            teams = search_teams(q)
            global_events = search_global_events(q)
            team_events = search_team_events(q)
            venues = search_venues(q)
            meetings = search_meetings(q)
    context = {
        'teams': teams,
        'global_events': global_events,
        'team_events': team_events,
        'venues': venues,
        'meetings': meetings,
        'q': q,
    }
    return render_to_response('site_search.html', context,
        RequestContext(request))


# pylint: disable-msg=C0103
def search_teams(q):
    from teams.models import Team
    from django.db.models import Q
    team_list = Team.objects.filter(Q(name__icontains=q) |
                                    Q(countries__name__icontains=q) |
                                    Q(city__icontains=q) |
                                    Q(languages__name__icontains=q)).order_by('name').distinct()
    return team_list


# pylint: disable-msg=C0103
def search_global_events(q):
    from events.models import GlobalEvent
    from django.db.models import Q
    global_event_list = GlobalEvent.objects.next_events()
    global_event_list = global_event_list.filter(Q(name__icontains=q)).distinct()
    return global_event_list


# pylint: disable-msg=C0103
def search_team_events(q):
    from events.models import TeamEvent
    from django.db.models import Q
    team_event_list = TeamEvent.objects.next_events()
    team_event_list = team_event_list.filter(Q(name__icontains=q) |
                                             Q(teams__name__icontains=q) |
                                             Q(venue__name__icontains=q) |
                                             Q(venue__city__icontains=q) |
                                             Q(venue__country__name__icontains=q) |
                                             Q(global_event__name__icontains=q)).distinct()
    return team_event_list


# pylint: disable-msg=C0103
def search_venues(q):
    from venues.models import Venue
    from django.db.models import Q
    venue_list = Venue.objects.filter(Q(name__icontains=q) |
                                      Q(country__name__icontains=q) |
                                      Q(city__icontains=q) |
                                      Q(address__icontains=q)).order_by('name').distinct()
    return venue_list


# pylint: disable-msg=C0103
def search_meetings(q):
    from meetings.models import TeamMeeting
    from django.db.models import Q
    meeting_list = TeamMeeting.objects.next_meetings()
    meeting_list = meeting_list.filter(Q(name__icontains=q) |
                                       Q(teams__name__icontains=q) |
                                       Q(agenda__title__icontains=q)).distinct()
    return meeting_list


def login_failure(request, message, status=403,
           template_name='login_failure.html',
           exception=None):
    """Render an error page to the user."""
    context = {
        'message': message,
        'exception': exception,
    }
    if isinstance(exception, MissingPhysicalMultiFactor):
        context['solution'] = 'Try logging in again using your Yubikey'
    elif isinstance(exception, MissingUsernameViolation):
        context['solution'] = 'You will need to create a ' \
                              '<a href="https://launchpad.net/people/+me">' \
                              'Launchpad profile</a> to use the LoCo Teams Directory'

    data = render_to_string(template_name, context,
        context_instance=RequestContext(request))
    return HttpResponse(data, status=status)
