# -*- coding: utf-8 -*-
from django import forms
from django.conf import settings
from django.utils.translation import ugettext as _
from django.core.urlresolvers import reverse

from .models import BaseEvent, GlobalEvent, TeamEvent, Attendee, TeamEventComment
from venues.models import Venue
from common.forms import RenderableMixin
from userprofiles.models import UserProfile
from .widgets import DateTimeWidget

import itertools


def grouped_user_list(teams):
    other_members, team_members = [], []
    for profile in UserProfile.objects.filter(user__groups__name__in=[t.lp_name for t in teams]):
        team_members.append((profile.id, str(profile)))
    for profile in UserProfile.objects.all().exclude(pk__in=[m[0] for m in team_members]):
        other_members.append((profile.id, str(profile)))

    return [('', '---------'),
            (_('Team members'), team_members),
            (_('Other users'), other_members)]


def validate_tag(tag):
    if tag.startswith('#'):
        tag = tag[1:]
    for delim in [" ", ",", "#"]:
        if delim in tag:
            raise forms.ValidationError(_(u'Please just set one tag.'))
    return tag


class BaseEventForm(forms.ModelForm, RenderableMixin):
    """
    a form to create/update a BaseEvent
    """
    class Meta:
        model = BaseEvent
        exclude = ('date_created')

    class Media:
        css = {'all': (
            settings.MEDIA_URL + 'jquery-ui/css/smoothness/jquery-ui.css',
        )}
        js = (
            settings.MEDIA_URL + 'jquery-ui/jquery-ui.js',
            settings.MEDIA_URL + 'js/events-ui.js',
        )

    def __init__(self, *args, **kargs):
        super(BaseEventForm, self).__init__(*args, **kargs)
        self.fields['date_begin'].widget = DateTimeWidget()
        self.fields['date_end'].widget = DateTimeWidget()

    def clean(self):
        begin = self.cleaned_data.get('date_begin')
        end = self.cleaned_data.get('date_end')
        if begin and end and begin > end:
            raise forms.ValidationError(_("Events can not end before they start."))
        return self.cleaned_data


class TeamEventForm(BaseEventForm):
    """
    a form to create/update a TeamEvent
    """
    class Meta(BaseEventForm.Meta):
        model = TeamEvent
        exclude = ('teams', 'date_created', 'created_by')

    def __init__(self, teams=[], *args, **kargs):
        super(TeamEventForm, self).__init__(*args, **kargs)
        self.teams = teams
        self.fields['contact'].choices = grouped_user_list(self.teams)
        self.fields['venue'].choices = self.grouped_venue_list()
        from common.widgets import PopupRelatedFieldWidgetWrapper
        self.fields['venue'].widget = PopupRelatedFieldWidgetWrapper(self.fields['venue'].widget, reverse('venue-new'))
        self.fields['global_event'].choices = [(e.id, str(e)) for e in GlobalEvent.objects.next_events().iterator()]
        self.fields['global_event'].choices.insert(0, ('', '---------'))
        self.initial['date_begin'] = self.instance.local_date_begin
        self.initial['date_end'] = self.instance.local_date_end

    def clean(self):
        venue = self.cleaned_data.get('venue')
        begin = self.cleaned_data.get('date_begin', None)
        end = self.cleaned_data.get('date_end', None)
        if venue is not None:
            if begin is not None and end is not None:
                self.cleaned_data['date_begin'] = venue.fromlocaltime(begin)
                self.cleaned_data['date_end'] = venue.fromlocaltime(end)
        elif self.teams:
            if begin is not None and end is not None:
                self.cleaned_data['date_begin'] = self.teams[0].fromlocaltime(begin)
                self.cleaned_data['date_end'] = self.teams[0].fromlocaltime(end)
        return self.cleaned_data

    def grouped_venue_list(self):
        """
        Returns a list of venues grouped by country
        in a way that Django will convert to OptGroups
        """
        venue_choices = [('', '---------')]
        previous_venues = []
        previous_choice = []
        country_choices = []
        current_country = ''

        # Have a look for previously used event venues
        team_events = TeamEvent.objects.filter(teams__in=self.teams)
        for event in team_events:
            if event.venue and event.venue not in previous_venues:
                previous_venues.append(event.venue)
                previous_choice.append((event.venue.id, str(event.venue)))

        # Add a previously used menu list if required.
        if previous_venues:
            venue_choices.insert(1, (_("Previously Used"), previous_choice))

        # Show the venues which belong the the 'current team' first. then show the other countries
        venues_team = Venue.objects.filter(country__team__in=self.teams).order_by('country', 'spr', 'city')
        venues_other = Venue.objects.all().exclude(country__team__in=self.teams).order_by('country', 'spr', 'city')
        venues = itertools.chain(venues_team, venues_other)

        # Ordered by country so we can tell when we're done with one
        # and starting another
        for venue in venues:
            # Ignore the venue if we've added it before.
            if venue in previous_venues:
                continue
            # If this venue has a different country, start a new group
            if venue.country != current_country:
                # Add the old group to the list if it's non-empty
                if len(country_choices) > 0:
                    venue_choices.append((current_country, country_choices))
                country_choices = []
                current_country = venue.country
            # Add venue to the current country group
            country_choices.append((venue.id, str(venue)))

        # Add the last country group to the list if it's non-empty
        if len(country_choices) > 0:
            venue_choices.append((current_country, country_choices))

        return venue_choices


class GlobalEventForm(BaseEventForm):
    """
    a form to create/update a GlobalEvent
    """
    class Meta(BaseEventForm.Meta):
        model = GlobalEvent

    def clean(self):
        self.cleaned_data['microbloghashtag'] = validate_tag(self.cleaned_data['microbloghashtag'])
        self.cleaned_data['pictag'] = validate_tag(self.cleaned_data['pictag'])
        return self.cleaned_data


class AttendeeRegistrationForm(forms.ModelForm, RenderableMixin):
    """
    a form to create/update an Attendee object
    """
    class Meta:
        model = Attendee
        exclude = ('attendee_profile', 'team_event')


class TeamEventCommentForm(forms.ModelForm, RenderableMixin):
    """
    a form to comment a team event
    """
    class Meta:
        model = TeamEventComment
        exclude = ('team_event', 'date_created', 'commenter_profile')
