# -*- coding: utf-8 -*-
from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _

from userprofiles.models import UserProfile
from teams.models import *
from venues.models import Venue
from common.shortcuts import queryset_sum, queryset_count
from common.mixins import LocalTimeMixin
from common.utils import slugify


from .managers import GlobalEventManager, TeamEventManager
from .fields import GuestsField


ATTENDEE_PROMISE_CHOICES = (
    ('sure', _('attending')),
    ('maybe', _('might be attending')),
    ('not', _('not attending')),
)

import pytz
import datetime


class BaseEvent(models.Model):
    """
    a simple basic event
    """
    name = models.CharField(max_length=150, verbose_name=_('Event Name'))
    date_begin = models.DateTimeField(help_text=_('Local date and time that the event starts'), verbose_name=_('Local Start Time'), db_index=True)
    date_end = models.DateTimeField(help_text=_('Local date and time that the event ends'), verbose_name=_('Local End Time'), db_index=True)
    description = models.TextField(help_text=_('Description of the Event'), blank=True, null=True)
    announce = models.URLField(verbose_name=_('Announcement'), help_text=_('URL for the announcement'), max_length=200, verify_exists=False, blank=True, null=True)
    date_created = models.DateTimeField(help_text=_('the date and time when the event was created'), default=datetime.datetime.now, db_index=True)

    def __unicode__(self):
        return self.name

    def as_ical(self, cal):
        """
        return a event as ical
        """
        event = cal.add('vevent')
        event.add('uid').value = str(self.id)
        event.add('dtstart').value = pytz.utc.localize(self.date_begin)
        event.add('dtend').value = pytz.utc.localize(self.date_end)
        event.add('categories').value = ['Ubuntu Loco Team Event']
        event.add('summary').value = self.name or ''
        event.add('description').value = self.description or ''

    def is_past(self):
        return self.date_end > datetime.datetime.today()


class GlobalEvent(BaseEvent):
    """
    a global event. other events can attend to a global event
    """
    microbloghashtag = models.CharField(max_length=50, help_text=_('Hash Tag used for microbloggers to mention this event'), verbose_name=_('Microblogging Hash Tag'), null=True, blank=True)
    pictag = models.CharField(max_length=50, help_text=_('tag used in web albums like; picasa, flickr, and more'), verbose_name=_('Picture Tag'), null=True, blank=True)
    channel = models.CharField(help_text=_('Channel for the Event hosted on the freenode IRC network. Ex. #ubuntu-locoteams'), max_length=25, verbose_name=_('Event Channel'), blank=True, null=True)
    activities = models.TextField(help_text=_('Activities to do during the Event'), blank=True, null=True)

    objects = GlobalEventManager()

    def __unicode__(self):
        return "%s %s" % (self.name, self.date_begin or "")

    @models.permalink
    def get_absolute_url(self):
        return ('global-event-detail', [str(self.id)])

    def get_participating(self):
        attendees = 0
        for attendee in Attendee.objects.filter(team_event__global_event=self, promise='sure'):
            attendees += attendee.guests + 1
        locos = len(Team.objects.filter(teamevent__global_event=self).distinct())
        countries = len(Country.objects.filter(team__teamevent__global_event=self).distinct())
        return _('%(attendees)d people from %(locos)d LoCos and %(countries)d countries are participating in this event!' % {'attendees': attendees,
                                                                                                                             'locos': locos,
                                                                                                                             'countries': countries})


class TeamEvent(BaseEvent, LocalTimeMixin):
    """
    a event of one or more teams
    """
    created_by = models.ForeignKey(User, null=True, blank=True)
    teams = models.ManyToManyField(Team, help_text=_('teams planning this event.'), verbose_name=_('Teams Participating'), db_index=True)
    contact = models.ForeignKey(UserProfile, verbose_name=_('Event Contact'), null=True, help_text=_('Contact person for this event.'))
    global_event = models.ForeignKey(GlobalEvent, help_text=_('this team event is part of the selected global event'), verbose_name=_('Related Global Event'), blank=True, null=True)
    venue = models.ForeignKey(Venue, help_text=_('Venue for the Event'), verbose_name=_('Event Venue'), blank=True, null=True)
    channel = models.CharField(help_text=_('Channel for the Event hosted on the freenode IRC network. Ex. #ubuntu-us-fl'), max_length=25, verbose_name=_('Event Channel'), blank=True, null=True)
    registration = models.URLField(verbose_name=_('Registration URL'), help_text=_('URL to register attendance (leave blank to use the built-in registration system)'), max_length=200, verify_exists=False, blank=True, null=True)

    objects = TeamEventManager()

    def __unicode__(self):
        return "%s %s %s" % (self.name, self.venue, self.date_begin or "")

    @models.permalink
    def get_absolute_url(self):
        return ('team-event-detail', [getattr(self.first_team(), 'lp_name', 'no-team'), str(self.id), slugify(u'%s' % self.name)])

    def as_ical(self, cal):
        """
        return a event as ical
        """
        event = cal.add('vevent')
        event.add('uid').value = str(self.id)
        event.add('dtstart').value = pytz.utc.localize(self.date_begin)
        event.add('dtend').value = pytz.utc.localize(self.date_end)
        event.add('categories').value = ['Ubuntu Loco Team Event']
        event.add('summary').value = u"%s" % self.name or ''
        event.add('description').value = u"%s" % self.description or ''
        event.add('location').value = u"%s" % self.venue or ''
        event.add('url').value = "http://loco.ubuntu.com%s" % self.get_absolute_url()
        if self.venue:
            event.add('geo').value = u"%s;%s" % (self.venue.latitude, self.venue.longitude)

    def get_tz(self):
        timezone = 'UTC'
        if self.venue:
            timezone = self.venue.tz
        else:
            try:
                timezone = self.teams.exclude(tz='UTC')[0].tz
            except IndexError:
                # There is not team with timezone set other than the default 'UTC'
                pass
        return timezone
    tz = property(get_tz)

    def get_local_begin(self):
        return self.tolocaltime(self.date_begin)

    def set_local_begin(self, local_begin):
        self.date_begin = self.fromlocaltime(local_begin)
    local_date_begin = property(get_local_begin)

    def get_local_end(self):
        return self.tolocaltime(self.date_end)

    def set_local_end(self, local_end):
        self.date_end = self.fromlocaltime(local_end)
    local_date_end = property(get_local_end, set_local_end)

    def attending(self):
        return self.attendee_set.filter(promise__exact='sure')

    def maybe_attending(self):
        return self.attendee_set.filter(promise__exact='maybe')

    def not_attending(self):
        return self.attendee_set.filter(promise__exact='not')

    def first_team(self):
        try:
            return self.teams.all()[0]
        except:
            #print 'Event %s has no team' % self.id
            return {'lp_name': 'no-team'}

    def is_attending(self, user):
        if user.is_authenticated():
            return self.attendee_set.filter(attendee_profile__user=user).count() > 0
        else:
            return False

    def total_attending(self):
        total_attending_guests = queryset_sum('guests', Attendee.objects.filter(team_event__id__exact=self.id).filter(promise="sure"))
        total_attending_registered = queryset_count('attendee_profile', Attendee.objects.filter(team_event__id__exact=self.id).filter(promise="sure"))
        return total_attending_guests + total_attending_registered

    def total_maybe_attending(self):

        total_maybe_attending_guests = queryset_sum('guests', Attendee.objects.filter(team_event__id__exact=self.id).filter(promise="maybe"))
        total_maybe_attending_registered = queryset_count('attendee_profile', Attendee.objects.filter(team_event__id__exact=self.id).filter(promise="maybe"))
        return total_maybe_attending_guests + total_maybe_attending_registered

    def comments_set(self):
        return self.teameventcomment_set.all().order_by('-date_created')


class TeamEventComment(models.Model):
    """
    a model for comments about a team event
    """
    team_event = models.ForeignKey(TeamEvent, help_text=_('the Team Event'))
    date_created = models.DateTimeField(help_text=_('Comment date and time'), default=datetime.datetime.now, db_index=True)
    commenter_profile = models.ForeignKey(UserProfile, null=True)
    comment = models.TextField(help_text=_('Enter your comment for this event.'), db_index=True)

    def __unicode__(self):
        return "Team Event: %s . Comment: %s" % (self.team_event.name, self.comment)

    def get_local_created(self):
        if self.team_event.venue:
            return self.team_event.venue.tolocaltime(self.date_created)
        else:
            return self.date_created

    def set_local_created(self, local_created):
        if self.team_event.venue:
            self.date_created = self.team_event.venue.fromlocaltime(local_created)
        else:
            self.date_created = local_created
    local_date_created = property(get_local_created, set_local_created)


class Attendee(models.Model):
    """
    a event attendee
    """

    team_event = models.ForeignKey(TeamEvent, help_text=_('the Team Event'))
    attendee_profile = models.ForeignKey(UserProfile, help_text=_('the name of the attendee'), null=True)
    promise = models.CharField(verbose_name=_('Attending Status'), max_length=50, choices=ATTENDEE_PROMISE_CHOICES)
    guests = GuestsField(verbose_name=_('Additional Guests'))

    class Meta:
        unique_together = ('team_event', 'attendee_profile')

    def __unicode__(self):
        return self.attendee_profile.user.username

try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["^events\.fields\.GuestsField"])
except:
    pass
