# -*- coding: utf-8 -*-
from django import forms
from django.conf import settings
from django.utils.translation import ugettext as _

from models import BaseMeeting, TeamMeeting, AgendaItem
from common.forms import RenderableMixin
from userprofiles.models import UserProfile
from events.widgets import DateTimeWidget

import urllib


def grouped_user_list(teams):
    other_members, team_members = [], []
    for profile in UserProfile.objects.filter(user__groups__name__in=[t.lp_name for t in teams]):
        team_members.append((profile.id, str(profile)))
    for profile in UserProfile.objects.all().exclude(pk__in=[m[0] for m in team_members]):
        other_members.append((profile.id, str(profile)))

    return [('', '---------'),
            (_('Team members'), team_members),
            (_('Other users'), other_members)]


class BaseMeetingForm(forms.ModelForm, RenderableMixin):
    """
    a form to create/update a BaseMeeting
    """
    class Meta:
        model = BaseMeeting
        exclude = ('date_created')

    class Media:
        css = {'all': (
                settings.MEDIA_URL + 'jquery-ui/css/smoothness/jquery-ui.css',
                )}
        js = (
              settings.MEDIA_URL + 'jquery-ui/jquery-ui.js',
              settings.MEDIA_URL + 'js/events-ui.js',
              )

    def __init__(self, *args, **kargs):
        super(BaseMeetingForm, self).__init__(*args, **kargs)
        self.fields['date_begin'].widget = DateTimeWidget()
        self.fields['date_end'].widget = DateTimeWidget()

    def clean(self):
        begin = self.cleaned_data.get('date_begin')
        end = self.cleaned_data.get('date_end')
        if begin and end and begin > end:
            raise forms.ValidationError("Meetings can not end before they start.")
        return self.cleaned_data


class TeamMeetingForm(BaseMeetingForm):
    """
    a form to create/update a TeamMeeting
    """
    class Meta(BaseMeetingForm.Meta):
        model = TeamMeeting
        fields = (
            'name',
            'date_begin',
            'date_end',
            'meeting_tz',
            'channel',
            'minutes',
            'chair',
            'logs'
        )
        exclude = ('teams', 'date_created')

    def __init__(self, teams=None, *args, **kargs):
        super(TeamMeetingForm, self).__init__(*args, **kargs)
        self.teams = teams
        if self.teams:
            self.fields['chair'].choices = grouped_user_list(self.teams)
        elif self.instance.teams:
            self.fields['chair'].choices = grouped_user_list(self.instance.teams.iterator())
        self.initial['date_begin'] = self.instance.local_date_begin
        self.initial['date_end'] = self.instance.local_date_end

    def clean(self):
        tz = self.cleaned_data.get('meeting_tz', 'team')
        if tz is not None:
            begin = self.cleaned_data.get('date_begin', None)
            end = self.cleaned_data.get('date_end', None)
            # If this is a new meeting it won't have any teams set at this point
            # So if we're using the team default timezone, we need to look it
            # up from the passed list of teams
            if tz == 'team' and not self.instance.pk:
                if self.teams:
                    local_to = self.teams[0]
                else:
                    # we don't have enough information to localize the time
                    return self.cleaned_data
            # If this is an existing meeting, or we specified a timezone for it
            # we just need to tell the instance what timezone to localize to
            else:
                self.instance.meeting_tz = tz
                local_to = self.instance

            if begin is not None and end is not None:
                self.cleaned_data['date_begin'] = local_to.fromlocaltime(begin)
                self.cleaned_data['date_end'] = local_to.fromlocaltime(end)
        return self.cleaned_data

    def save(self):
        start_date = self.cleaned_data['date_begin']
        if self.instance.logs is None or "http://irclogs.ubuntu.com" in self.instance.logs:
            self.instance.logs = 'http://irclogs.ubuntu.com/%(date)s/%(channel)s.html#t%(time)s' % {
                'date': start_date.strftime('%Y/%m/%d'),
                'channel': urllib.quote(u'%s' % self.cleaned_data['channel'].encode('ascii', 'ignore')),
                'time': start_date.strftime('%H:%M')
            }
        return super(TeamMeetingForm, self).save()


class AgendaItemForm(forms.ModelForm, RenderableMixin):

    class Meta:
        model = AgendaItem
        exclude = ('meeting', 'created_date')

    def __init__(self, *args, **kargs):
        super(AgendaItemForm, self).__init__(*args, **kargs)
        parent_choices = AgendaItem.objects.filter(meeting=self.instance.meeting).exclude(pk=self.instance.id)
        self.fields['parent'].choices = [('', '')] + [(item.id, unicode(item)) for item in parent_choices if item not in self.instance.descendents]
        if self.instance:
            meeting = self.instance.meeting.teammeeting
            self.fields['owner'].choices = grouped_user_list(meeting.teams.iterator())


class AgendaItemFormSet(forms.models.BaseInlineFormSet):

    def __init__(self, *args, **kargs):
        super(AgendaItemFormSet, self).__init__(*args, **kargs)
        self.node_tree = {}

    def add_fields(self, form, index):
        super(AgendaItemFormSet, self).add_fields(form, index)
        self.fields['node_id'] = forms.IntegerField(label=_(u'Node'),
                        initial=index,
                        required=False)
        self.node_tree[form.instance.id] = index
        self.fields['parent_node'] = forms.IntegerField(label=_(u'Parent Node'),
                        initial=self.node_tree.get(form.instance.parent, None),
                        required=False)

    def save(self):
        pass

    def as_tree(self):
        pass
