from django.template import RequestContext
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response, redirect, get_object_or_404
from django.contrib.auth.decorators import login_required
from django.utils.translation import ugettext as _
from django.core.urlresolvers import reverse

from meetings.models import TeamMeeting, AgendaItem
from teams.models import Team

from forms import TeamMeetingForm, AgendaItemForm

from common import launchpad
from common.forms import FilterHistoryList

from userprofiles.models import UserProfile
import datetime
import vobject


def meeting_list(request):
    """
    a list with all meetings (teammeetings)
    """
    team_meeting_list = TeamMeeting.objects.next_meetings()
    is_on_lc = launchpad.is_user_on_loco_council(request.user)
    context = {
        'team_meeting_list': team_meeting_list,
    'user_is_on_lc': is_on_lc,
    }
    return render_to_response('meetings/meeting_list.html', context,
                  RequestContext(request))


def meeting_list_ical(meetings, name):
    """Return any list meetings as an ical"""
    filename = "%s.ics" % name.replace(' ', '-').lower()
    response = HttpResponse(mimetype='text/calendar')
    response['Content-Disposition'] = 'attachment; filename=%s' % filename.encode('ascii', 'replace')
    calendar = vobject.iCalendar()
    calendar.add('prodid').value = '-//loco.ubuntu.com//EN'
    calendar.add('version').value = '2.0'
    calendar.add('calscale').value = 'GREGORIAN'
    calendar.add('method').value = 'PUBLISH'
    calendar.add('x-wr-timezone').value = 'UTC'
    calendar.add('x-wr-calname').value = name
    calendar.add('x-wr-caldesc').value = name
    for meeting in meetings:
        meeting.as_ical(calendar)
    response.write(calendar.serialize())
    return response


def meeting_ical(request, team_slug, team_meeting_id):
    """
    Return a ical list with a single meetings in ical format.
    """
    team_meeting_object = get_object_or_404(TeamMeeting, pk=team_meeting_id)
    return meeting_list_ical([team_meeting_object], team_meeting_object.name)


def team_meeting_list_ical(request, team_slug):
    """
    Return a ical list with a single team meetings in ical format.
    """
    team = get_object_or_404(Team, lp_name=team_slug)
    team_meeting = TeamMeeting.objects.filter(teams__lp_name=team_slug)
    return meeting_list_ical(team_meeting, team.name)


def teams_meeting_list_ical(request):
    """
    Return a ical list with team meetings in ical format.
    """
    return meeting_list_ical(TeamMeeting.objects.all(), 'All Ubuntu Team Meetings')


def meeting_history_list(request):
    """
    a list with all historical meetings (teammeetings)
    """
    team_meeting_list = TeamMeeting.objects.history_meetings()

    if request.method == 'POST':
        form = FilterHistoryList(request.POST)
        if form.is_valid():

            filtering = form.cleaned_data['filtering']

            scope = form.cleaned_data['scope']

            if u'my' in scope:
                filtering_date = datetime.datetime.strptime(filtering, "%Y-%m")

                team_meeting_list = TeamMeeting.objects.history_meetings().filter(date_begin__year=filtering_date.year, date_begin__month=filtering_date.month)
            elif u'y' in scope:
                filtering_date = datetime.datetime.strptime(filtering, "%Y-%m")

                team_meeting_list = TeamMeeting.objects.history_meetings().filter(date_begin__year=filtering_date.year)
        else:
            team_meeting_list = TeamMeeting.objects.history_meetings()

    else:
        form = FilterHistoryList()
        team_meeting_list = TeamMeeting.objects.history_meetings()

    context = {
        'form': form,
        'team_meeting_list': team_meeting_list,
    }
    return render_to_response('meetings/meeting_history_list.html', context,
                  RequestContext(request))


#################################################################
# Team Meetings
#################################################################

def team_meeting_detail(request, team_slug, team_meeting_id):
    team_meeting_object = get_object_or_404(TeamMeeting, pk=team_meeting_id)
    is_member = False
    for team_object in team_meeting_object.teams.all():
        is_member = is_member or launchpad.is_team_member(request.user, team_object)

    context = {
        'team_meeting_object': team_meeting_object,
        'user_is_team_member': is_member,
    }
    return render_to_response('meetings/team_meeting_detail.html',
                                         context, RequestContext(request))


@login_required
def team_meeting_delete(request, team_slug, team_meeting_id):
    """
    delete a meeting
    """
    team_meeting_object = get_object_or_404(TeamMeeting, pk=team_meeting_id)

    is_on_lc = launchpad.is_user_on_loco_council(request.user)
    #check if user is admin or owner of a team
    is_contact = False
    for team in team_meeting_object.teams.all():
        if launchpad.is_admin_or_owner(request.user.username, team):
            is_contact = True
            break

    if is_on_lc or is_contact:
        if request.method == 'POST':
            team_meeting_object.delete()
            request.user.message_set.create(message=_('Team Meeting removed.'))
            return redirect('meeting-list')
        else:
            context = {'team_meeting_object': team_meeting_object}
            return render_to_response('meetings/team_meeting_delete_confirm.html', context, RequestContext(request))

    else:
        request.user.message_set.create(message='%s %s' % (_('You can not remove this team meeting.'), _('You are not an admin/owner of the Launchpad team or on the LoCo Council.')))
        return redirect(team_meeting_object)


@login_required
def team_meeting_select(request):
    teams = Team.objects.filter(lp_name__in=[g.name for g in request.user.groups.all()])
    if len(teams) == 0:
        request.user.message_set.create(message='%s' % (_('You are not a member of any LoCo Teams.')))
        return redirect('meeting-list')
    elif len(teams) == 1:
        url = reverse('team-meeting-new', args=[teams[0].lp_name])
        return HttpResponseRedirect(url)
    else:
        context = {'teams': teams}
        return render_to_response('meetings/team_meeting_new_select.html', context, RequestContext(request))


def team_meetings_rss(request, team_slug):
    return redirect('/feeds/meetings/%s/' % team_slug, permanent=True)


@login_required
def team_meeting_new(request, team_slug):
    """
    new team meeting
    """
    team_object = get_object_or_404(Team, lp_name=team_slug)

    is_on_lc = launchpad.is_user_on_loco_council(request.user)
    is_member = launchpad.is_team_member(request.user, team_object)

    team_meeting_object = TeamMeeting(chair=team_object.owner_profile,
                                      channel=team_object.irc_chan)
    if is_on_lc or is_member:
        if request.method == 'POST':
            form = TeamMeetingForm(data=request.POST, teams=[team_object], instance=team_meeting_object)
            if form.is_valid():
                team_meeting = form.save()
                team_meeting.teams.add(team_object)
                return HttpResponseRedirect(team_meeting_object.get_absolute_url())
        else:
            form = TeamMeetingForm(teams=[team_object], instance=team_meeting_object)

        context = {
            'team_object': team_object,
            'form': form,
        }
        return render_to_response('meetings/team_meeting_new.html',
                                         context, RequestContext(request))
    else:
        # XXX: Once we move to a new ACL system, this needs fixing.
        request.user.message_set.create(message='%s %s' % (_('You can not add a new meeting for this team.'), _('You are not a member of the team or on the LoCo Council.')))
        return redirect(team_object)


@login_required
def team_meeting_copy(request, team_slug, team_meeting_id):
    """
    copy team meeting
    """

    team_meeting = get_object_or_404(TeamMeeting, pk=team_meeting_id)
    team_object = team_meeting.teams.all()[0]

    is_on_lc = launchpad.is_user_on_loco_council(request.user)
    is_member = launchpad.is_team_member(request.user, team_object)

    if is_on_lc or is_member:
        if request.method == 'POST':
            form = TeamMeetingForm(instance=team_meeting, data=request.POST, teams=team_meeting.teams.all())
            team_meeting.id = team_meeting.pk = None
            if form.is_valid():
                team_meeting = form.save()
                team_meeting.teams.add(team_object)
                team_meeting_id = team_meeting.id
                return HttpResponseRedirect(team_meeting.get_absolute_url())
        else:
            form = TeamMeetingForm(instance=team_meeting, teams=team_meeting.teams.all())

        context = {
            'team_object': team_object,
            'form': form,
        }
        return render_to_response('meetings/team_meeting_new.html',
                                         context, RequestContext(request))
    else:
        # XXX: Once we move to a new ACL system, this needs fixing.
        request.user.message_set.create(message='%s %s' % (_('You can not add a new meeting for this team.'), _('You are not a member of the team or on the LoCo Council.')))
        return redirect(team_object)


@login_required
def team_meeting_update(request, team_slug, team_meeting_id):
    """
    update team meeting
    """
    team_meeting_object = get_object_or_404(TeamMeeting, pk=team_meeting_id)
    #check if user is admin or owner of a team
    is_member = False
    for team in team_meeting_object.teams.all():
        if launchpad.is_team_member(request.user, team):
            is_member = True
            break

    is_on_lc = launchpad.is_user_on_loco_council(request.user)

    if is_on_lc or is_member:
        if request.method == 'POST':
            form = TeamMeetingForm(data=request.POST, instance=team_meeting_object)
            if form.is_valid():
                form.save()
                request.user.message_set.create(message=_('Team meeting updated.'))
                return redirect(team_meeting_object)
        else:
            form = TeamMeetingForm(instance=team_meeting_object)

        context = {
            'form': form,
        }
        return render_to_response(
            'meetings/team_meeting_update.html',
            context,
            RequestContext(request)
        )
    else:
        request.user.message_set.create(message='%s %s' % (_('You can not update this team meeting.'), _('You are not a member of the team or on the LoCo Council.')))
        return redirect(team_meeting_object)


@login_required
def agenda_item_new(request, team_meeting_id, agenda_item_id=None):
    """
    new agenda item
    """
    team_meeting_object = get_object_or_404(TeamMeeting, pk=team_meeting_id)
    try:
        user = UserProfile.objects.get(user=request.user)
        agenda_item_object = AgendaItem(
            meeting=team_meeting_object,
            owner=user,
            created_date=datetime.datetime.now()
        )
    except UserProfile.DoesNotExist:
        agenda_item_object = AgendaItem(
            meeting=team_meeting_object,
            created_date=datetime.datetime.now()
        )

    if agenda_item_id is not None:
        agenda_item_object.parent = get_object_or_404(AgendaItem, pk=agenda_item_id)
    is_member = False
    for team in team_meeting_object.teams.all():
        if launchpad.is_team_member(request.user, team):
            is_member = True
            break
    is_on_lc = launchpad.is_user_on_loco_council(request.user)

    if is_on_lc or is_member:
        if request.method == 'POST':
            form = AgendaItemForm(data=request.POST, instance=agenda_item_object)
            if form.is_valid():
                agenda_item_object = form.save()
                request.user.message_set.create(message=_('Meeting agenda updated.'))
                return redirect(team_meeting_object)
        else:
            form = AgendaItemForm(instance=agenda_item_object)

        context = {
            'team_meeting_object': team_meeting_object,
            'form': form,
        }
        return render_to_response('meetings/agenda_item_new.html',
                                         context, RequestContext(request))
    else:
        # XXX: Once we move to a new ACL system, this needs fixing.
        request.user.message_set.create(message='%s %s' % (_('You can not add a new agenda item for this team meeting.'), _('You are not a member of the team or on the LoCo Council.')))
        return redirect(team_meeting_object)


@login_required
def agenda_item_update(request, team_meeting_id, agenda_item_id):
    """
    update agenda item
    """
    team_meeting_object = get_object_or_404(TeamMeeting, pk=team_meeting_id)
    agenda_item_object = get_object_or_404(AgendaItem, pk=agenda_item_id)
    #check if user is admin or owner of a team
    is_member = False
    for team in team_meeting_object.teams.all():
        if launchpad.is_team_member(request.user, team):
            is_member = True
            break

    is_on_lc = launchpad.is_user_on_loco_council(request.user)

    if is_on_lc or is_member:
        if request.method == 'POST':
            form = AgendaItemForm(data=request.POST, instance=agenda_item_object)
            if form.is_valid():
                form.save()
                request.user.message_set.create(message=_('Meeting agenda updated.'))
                return redirect(team_meeting_object)
        else:
            form = AgendaItemForm(instance=agenda_item_object)

        context = {
            'team_meeting_object': team_meeting_object,
            'agenda_item_object': agenda_item_object,
            'form': form,
        }
        return render_to_response('meetings/agenda_item_update.html',
                                         context, RequestContext(request))
    else:
        request.user.message_set.create(message='%s %s' % (_('You can not update this team meeting agenda.'), _('You are not a member of the team or on the LoCo Council.')))
        return redirect(team_meeting_object)


@login_required
def agenda_item_delete(request, team_meeting_id, agenda_item_id):
    """
    delete an agenda item
    """
    team_meeting_object = get_object_or_404(TeamMeeting, pk=team_meeting_id)
    agenda_item_object = get_object_or_404(AgendaItem, pk=agenda_item_id)
    #check if user is admin or owner of a team
    is_member = False
    for team in team_meeting_object.teams.all():
        if launchpad.is_team_member(request.user, team):
            is_member = True
            break

    is_on_lc = launchpad.is_user_on_loco_council(request.user)

    if is_on_lc or is_member:
        if request.method == 'POST':
            agenda_item_object.delete()
            request.user.message_set.create(message=_('Agenda Item removed.'))
            return redirect(team_meeting_object)
        else:
            context = {
                'team_meeting_object': team_meeting_object,
                'agenda_item_object': agenda_item_object}
            return render_to_response('meetings/agenda_item_delete_confirm.html', context, RequestContext(request))

    else:
        request.user.message_set.create(message='%s %s' % (_('You can not remove this agenda item.'), _('You are not an admin/owner of the Launchpad team or on the LoCo Council.')))
        return redirect(team_meeting_object)
