# Django settings for loco_directory project.

import os
import sys
from common import utils
import logging

st_handler = logging.StreamHandler(sys.stderr)
formatter = logging.Formatter("[%(asctime)s] - %(levelname)s - %(module)s - %(message)s")
st_handler.setFormatter(formatter)

log = logging.getLogger(__name__)
log.addHandler(st_handler)
log.setLevel(logging.DEBUG)

uw_import = False
try:
    import ubuntu_website
    uw_import = True
except ImportError:
    try:
        import bzr_apps
        pass
    except:
        if not 'init' in sys.argv:
            log.error('You will need to run ./manage.py init to make ' \
                      'the LoCo Team Portal fully work.')

# Check for necessary modules
modules = {
            'pytz': 'python-tz',
            'vobject': 'python-vobject',
          }

for module, package in modules.items():
    try:
        __import__(module)
    except ImportError:
        log.error('Missing library: %s, Install package: %s\n' % (module, package))
        sys.exit(1)

PROJECT_PATH = os.path.dirname(os.path.abspath(__file__))

DEBUG = True
TEMPLATE_DEBUG = DEBUG
STATIC_SERVE = True
USE_DEBUG_TOOLBAR = False

ADMINS = (
    ('somebody', 'somebody@example.com'),
)

MANAGERS = ADMINS
LP_PROJECT_NAME = 'loco-directory'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': 'ltp',
        'USER': 'postgres',
        'PASSWORD': '',
        'HOST': 'localhost',
        'PORT': '',
    },
}

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'UTC'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# Absolute path to the directory that holds media.
# Example: "/home/media/media.lawrence.com/"
MEDIA_ROOT = os.path.join(PROJECT_PATH, 'media')

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash if there is a path component (optional in other cases).
# Examples: "http://media.lawrence.com", "http://example.com/media/"
MEDIA_URL = '/media/'

# URL prefix for admin media -- CSS, JavaScript and images. Make sure to use a
# trailing slash.
# Examples: "http://foo.com/media/", "/media/".
ADMIN_MEDIA_PREFIX = '/media/admin/'

# Make this unique, and don't share it with anybody.
SECRET_KEY = ''

TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
)

# list with template context processors. see http://docs.djangoproject.com/en/dev/ref/settings/#template-context-processors
TEMPLATE_CONTEXT_PROCESSORS = (
    "django.contrib.auth.context_processors.auth",
    "django.core.context_processors.debug",
    "django.core.context_processors.i18n",
    "django.core.context_processors.media",
    "django.core.context_processors.csrf",
    "common.context_processors.loco_version",
    "common.context_processors.google_api_key",
    "common.context_processors.flickr_api_key",
    "common.context_processors.pixie_api_key",
    "common.context_processors.login_redirect",
    "common.context_processors.url_base",
    "common.context_processors.site_search",
    "common.context_processors.languages_menu",
)

ROOT_URLCONF = 'loco_directory.urls'

TEMPLATE_DIRS = (
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
    os.path.join(PROJECT_PATH, 'templates'),
)

if uw_import:
    TEMPLATE_CONTEXT_PROCESSORS += (
        "ubuntu_website.media_processor",
        "ubuntu_website.popup_check",
    )
    TEMPLATE_DIRS += (
        ubuntu_website.TEMPLATE_DIR,
    )

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.admin',
    'common',
    'teams',
    'venues',
    'events',
    'meetings',
    'articles',
    'userprofiles',
    'django_openid_auth',
    'south',
]

AUTHENTICATION_BACKENDS = (
    'django_openid_auth.auth.OpenIDBackend',
    'django.contrib.auth.backends.ModelBackend',
)

AUTH_PROFILE_MODULE = 'userprofiles.UserProfile'


OPENID_STRICT_USERNAMES = True
OPENID_FOLLOW_RENAMES = True
OPENID_SREG_REQUIRED_FIELDS = ['email', ]


# Should users be created when new OpenIDs are used to log in?
OPENID_CREATE_USERS = True

# Can we reuse existing users?
OPENID_REUSE_USERS = True

# When logging in again, should we overwrite user details based on
# data received via Simple Registration?
OPENID_UPDATE_DETAILS_FROM_SREG = True

# If set, always use this as the identity URL rather than asking the
# user.  This only makes sense if it is a server URL.
OPENID_SSO_SERVER_URL = 'https://login.ubuntu.com/'

# Automatically map lp team membership to ld group membership
OPENID_LAUNCHPAD_TEAMS_MAPPING_AUTO = True

# Tell django.contrib.auth to use the OpenID signin URLs.
LOGIN_URL = '/openid/login'
LOGIN_REDIRECT_URL = '/'


FLICKR_API_KEY = '8c969a1e8a49629bb89b411930ab1cc8'
PIXIE_API_KEY = '1dcc8233-a6da-46b0-acb0-4e29aa612b30'

# Manage apps from bzr branches
try:
    import bzr_apps
    INSTALLED_APPS.append('bzr_apps')
except:
    pass

BZR_APPS = {
    ## ubuntu-django-foundations app management
    'bzr_apps': ('http://bazaar.launchpad.net/~django-foundations-dev/ubuntu-django-foundations/bzr_apps', '8'),

    ## ubuntu-website supplied templates and styles
    'ubuntu_website': ('http://bazaar.launchpad.net/~ubuntu-community-webthemes/ubuntu-community-webthemes/light-django-theme', '50'),

    ## twidenash supplied microblog embedding javascript
    'media/js/twidenash': ('http://bazaar.launchpad.net/~django-foundations-dev/twidenash/2.0', '4'),
}

try:
    from local_settings import *
except ImportError:
    logging.warning("No local_settings.py were found. See INSTALL for instructions.")


if DEBUG and USE_DEBUG_TOOLBAR:
    MIDDLEWARE_CLASSES += (
        'debug_toolbar.middleware.DebugToolbarMiddleware',
    )

    INSTALLED_APPS += (
        'debug_toolbar',
    )

    DEBUG_TOOLBAR_PANELS = (
        'debug_toolbar.panels.version.VersionDebugPanel',
        'debug_toolbar.panels.timer.TimerDebugPanel',
        'debug_toolbar.panels.settings_vars.SettingsVarsDebugPanel',
        'debug_toolbar.panels.headers.HeaderDebugPanel',
        'debug_toolbar.panels.request_vars.RequestVarsDebugPanel',
        'debug_toolbar.panels.sql.SQLDebugPanel',
        'debug_toolbar.panels.template.TemplateDebugPanel',
        'debug_toolbar.panels.cache.CacheDebugPanel',
        'debug_toolbar.panels.logger.LoggingPanel',
    )

    DEBUG_TOOLBAR_CONFIG = {
        'INTERCEPT_REDIRECTS': False,
    }

    def show_toolbar(request):
        return True
    SHOW_TOOLBAR_CALLBACK = show_toolbar
