from django.test import TestCase
from django.contrib.auth.models import User, Group
from django.core.urlresolvers import reverse
from urllib import urlencode
from django_openid_auth.models import UserOpenID
from teams.models import *
from common import launchpad


class TestBase(TestCase):

    def setUp(self):
        # setup test users
        # user foo
        self.user_foo = User.objects.create(
            username='foo',
        )
        self.user_foo.set_password('test')
        self.user_foo.save()

        self.useropenid_foo = UserOpenID(user=self.user_foo,
            claimed_id='http://example.com/identity_foo',
            display_id='http://example.com/identity_foo')
        self.useropenid_foo.save()
        self.userprofile_foo = self.user_foo.get_profile()

        # user bar
        self.user_bar = User.objects.create(
            username='bar',
        )
        self.user_bar.set_password('test')
        self.user_bar.save()
        self.useropenid_bar = UserOpenID(user=self.user_bar,
            claimed_id='http://example.com/identity_bar',
            display_id='http://example.com/identity_bar')
        self.useropenid_bar.save()
        self.userprofile_bar = self.user_bar.get_profile()

        # setup test country
        self.country = Country.objects.create(name='Test Country')

        # setup test team
        self.team = Team.objects.create(
            lp_name='test-team',
            name='Test Team',
            owner_profile=self.userprofile_foo,
        )

        # adding team to county
        self.team.countries.add(self.country)
        self.team_group = Group.objects.create(
            name=self.team.lp_name,
        )

        # adding user foo to team_group
        self.user_foo.groups.add(self.team_group)
        self.user_foo.save()


class ACLChecksTestCase(TestBase):

    def testAdminIsATeamChecks(self):
        """Test if check for a more complicated LoCo Team setup works
        """
        team = Team.objects.get(lp_name="test-team")
        self.assertEquals(launchpad.is_admin_or_owner(self.user_foo.username, team), True)
        self.assertEquals(launchpad.is_admin_or_owner(self.user_bar.username, team), False)


class EditTeamTestCase(TestBase):
    """
    test if team update works correct
    """
    def test_anonymous_user(self):
        """
        test the team update as anonymous user (not logged in)
        """
        #try to access team-update page as not logged in user

        data = {'next': '/teams/test-team/edit/'}
        response = self.client.get(reverse('team-edit', args=['test-team']), follow=True)
        self.assertRedirects(response, '/openid/login/?%s' % urlencode(data), status_code=302, target_status_code=200)

        #try to update team as not logged in user
        response = self.client.post(reverse('team-edit', args=['test-team']), {'Country': 'Germany', 'forum_url': 'http://ubuntuusers.de'}, follow=True)
        self.assertRedirects(response, '/openid/login/?%s' % urlencode(data), status_code=302, target_status_code=200)

    def test_user_with_no_rights(self):
        """
        test the team update with a user which has no rights
        """
        pass
        #FIXME How to use a user with no right with openid?

    def test_user_loco_council(self):
        """
        test the team update with a user which is a member of loco council
        """
        pass
        #FIXME How to use a user with this rights with openid?

    def test_user_admin_or_owner(self):
        """
        test the team update with a user which is owner or admin
        """
        pass
        #FIXME How to use a user with this rights with openid?


class DisplayNameTestCase(TestBase):

    def test_override_name(self):
        # check without override_name
        self.assertEquals(self.team.display_name, 'Test Team')

        # check wih override_name
        self.team.override_name = 'Override Name'
        self.team.save()
        self.assertEquals(self.team.display_name, 'Override Name')

    def test_display_name_in_list(self):
        self.team.override_name = 'Override Name'
        self.team.save()

        response = self.client.get('/teams/')
        self.assertEquals(response.status_code, 200)
        self.assertContains(response, 'Override Name')

    def test_details_page(self):
        'Make sure the proper name is display on the team details page'
        self.team.override_name = 'Override Name'
        self.team.save()

        response = self.client.get('/teams/test-team/')
        self.assertEquals(response.status_code, 200)

        self.assertContains(response, 'Override Name', 0)  # No instances of override_name
        self.assertContains(response, 'Test Team', 4)  # One instance of the team name
