# -*- coding: utf-8 -*-
from django import http
from django.template import RequestContext
from django.utils.translation import ugettext
from django.core import serializers
from django.contrib.auth.models import User
from django.shortcuts import render_to_response, redirect, get_object_or_404
from django.contrib.auth.decorators import login_required
from django.http import Http404

from common.utils import simple_iterator
from common import launchpad
from teams.models import Continent, Team, countries_without_continent, countries_without_continent_have_teams, teams_without_country
from teams.utils import merge_teams
from events.models import TeamEvent
from meetings.models import TeamMeeting
from articles.models import Article

import forms
import datetime
import vobject


def reapprovals(request):
    return redirect('/loco-council/verified/')

def verified(request):
    teams = filter(lambda a: a.approved and a.expires_date != None,
                   Team.objects.all().order_by('expires_date', 'name'))
    today = datetime.date.today()
    half_a_year = datetime.timedelta(days=180)
# XXX: we have no idea why, but on loco.ubuntu.com we are dealing with
# datetime.datetime instead of datetime.date - bug 581641
    if type(teams[0].expires_date) == datetime.datetime:
        next_teams = filter(lambda a: a.expires_date.date() <= today + half_a_year, teams)
        later_teams = filter(lambda a: a.expires_date.date() > today + half_a_year, teams)
    else:
        next_teams = filter(lambda a: a.expires_date <= today + half_a_year, teams)
        later_teams = filter(lambda a: a.expires_date > today + half_a_year, teams)
    context = {
        'next_teams': next_teams,
        'later_teams': later_teams,
    }
    return render_to_response('teams/team_reapprovals.html',
                context, RequestContext(request))


def loco_council(request):

    team = get_object_or_404(Team, lp_name="ubuntu-lococouncil")
    articles = Article.objects.filter(source__team=team)[:5]
    is_member = False
    is_admin = False
    if request.user.is_authenticated():
        is_member = launchpad.is_team_member(request.user, team)
        is_admin = launchpad.is_admin_or_owner(request.user.username, team) or launchpad.is_user_on_loco_council(request.user)
    team_meeting_list = team.teammeeting_set.next_meetings()
    members = User.objects.filter(groups__name='ubuntu-lococouncil')
    context = {
        'team': team,
        'team_meeting_list': team_meeting_list,
        'articles': articles,
        'members': members,
        'is_member': is_member,
        'is_admin': is_admin,
    }
    return render_to_response('loco_council.html', context,
        RequestContext(request))


@login_required
def loco_council_edit(request):
    team_object = get_object_or_404(Team, lp_name="ubuntu-lococouncil")

    #check if user has needed rights
    is_on_lc = launchpad.is_user_on_loco_council(request.user)
    if not is_on_lc:
        request.user.message_set.create(message=ugettext('You are not allowed to make changes to the team details.'))
        return redirect('/loco-council/')

    if request.method == "POST":
        form = forms.EditLoCoCouncilForm(data=request.POST, instance=team_object)

        if form.is_valid():
            form.save()
            request.user.message_set.create(message=ugettext('Thanks, your team\'s details have been saved.'))
            return redirect('/loco-council/')
        else:
            request.user.message_set.create(message=ugettext('Recent changes to your team\'s details could not be saved.'))
    else:
        form = forms.EditLoCoCouncilForm(instance=team_object)

    return render_to_response('lc_edit.html',
                    {'form': form,
                     'team': team_object,
                    }, RequestContext(request))


def as_ical(objects, name):
    """Return any list events as an ical"""
    filename = "%s.ics" % name.replace(' ', '-').lower()
    response = http.HttpResponse(mimetype='text/calendar')
    response['Content-Disposition'] = 'attachment; filename=%s' % filename.encode('ascii', 'replace')
    calendar = vobject.iCalendar()
    calendar.add('prodid').value = '-//loco.ubuntu.com//EN'
    calendar.add('calscale').value = 'GREGORIAN'
    calendar.add('method').value = 'PUBLISH'
    calendar.add('x-wr-timezone').value = 'UTC'
    calendar.add('x-wr-calname').value = name
    calendar.add('x-wr-caldesc').value = name
    for o in objects:
        o.as_ical(calendar)
    response.write(calendar.serialize())

    return response


def team_list(request):
    continents = Continent.objects.all().order_by('name')
    for continent in continents:
        continent.continent_related_teams = continent.related_teams

    all_countries_without_continent = countries_without_continent().order_by('name')
    for country in all_countries_without_continent:
        country.country_without_continent_related_teams = country.related_teams

    context = {
        'continents': continents,
        'countries_without_continent': all_countries_without_continent,
        'countries_without_continent_have_teams': countries_without_continent_have_teams(),
        'teams_without_country': teams_without_country().order_by('name'),
        'colcycle': simple_iterator('col_left', 'col_right'),
    }
    return render_to_response('teams/team_list.html', context,
                  RequestContext(request))


def get_events_and_meetings(team_object):
    team_events = list(TeamEvent.objects.filter(teams=team_object))
    team_meetings = list(TeamMeeting.objects.filter(teams=team_object))
    team_events.extend(team_meetings)
    return team_events


def team_ical(request, team_slug):
    """
    Return a ical list with the events and meetings in ical format.
    """
    team_object = get_object_or_404(Team, lp_name=team_slug)
    return as_ical(get_events_and_meetings(team_object), team_object.name)


def team_event_list(request, team_slug):
    """
    list with all team events in present and future for the given team
    """
    team_object = get_object_or_404(Team, lp_name=team_slug)
    team_event_list = team_object.teamevent_set.next_events()
    is_member = False
    if request.user.is_authenticated():
        is_member = launchpad.is_team_member(request.user, team_object)
    context = {
        'team_object': team_object,
        'team_event_list': team_event_list,
        'is_member': is_member,
    }
    return render_to_response('teams/team_event_list.html', context,
                  RequestContext(request))


def team_event_history(request, team_slug):
    """
    list with all team events in past for the given team
    """
    team = get_object_or_404(Team, lp_name=team_slug)
    team_events = team.teamevent_set.history_events()
    context = {
        'team': team,
        'team_events': team_events,
    }
    return render_to_response('teams/team_event_history.html', context,
                  RequestContext(request))


def team_meeting_list(request, team_slug):
    """
    list with all team meetings in present and future for the given team
    """
    team_object = get_object_or_404(Team, lp_name=team_slug)
    team_meeting_list = team_object.teammeeting_set.next_meetings()
    context = {
        'team_object': team_object,
        'team_meeting_list': team_meeting_list,
    }
    return render_to_response('teams/team_meeting_list.html', context,
                  RequestContext(request))


def team_meeting_history(request, team_slug):
    """
    list with all team meetings in past for the given team
    """
    team = get_object_or_404(Team, lp_name=team_slug)
    team_meetings = team.teammeeting_set.history_meetings()
    context = {
        'team': team,
        'team_meetings': team_meetings,
    }
    return render_to_response('teams/team_meeting_history.html', context,
                  RequestContext(request))


def team_detail(request, team_slug):
    if team_slug == 'ubuntu-lococouncil':
        return redirect('/loco-council/')
    team_object = get_object_or_404(Team, lp_name=team_slug)
    articles = Article.objects.filter(source__team=team_object)[:5]
    is_member = False
    is_admin = False
    if request.user.is_authenticated():
        is_member = launchpad.is_team_member(request.user, team_object)
        is_admin = launchpad.is_admin_or_owner(request.user.username, team_object) or launchpad.is_user_on_loco_council(request.user)
    context = {
        'team': team_object,
        'articles': articles,
        'is_member': is_member,
        'is_admin': is_admin,
    }
    return render_to_response('teams/team_detail.html',
                                         context, RequestContext(request))


@login_required
def select_other_team(request, team_slug):
    team_object = get_object_or_404(Team, lp_name=team_slug)

    #check if user has needed rights
    is_admin = launchpad.is_admin_or_owner(request.user.username, team_object)
    if not is_admin:
        request.user.message_set.create(message=ugettext('You are not allowed to merge this team with another one.'))
        return redirect(team_object)
    team_list = Team.objects.all().order_by('name')
    other_teams = team_list.exclude(id=team_object.id)
    return render_to_response('teams/select_other_team.html',
                              {'team_object': team_object,
                               'other_teams': other_teams,
                               'user': request.user,
                              }, RequestContext(request))


@login_required
def team_merge(request, team_slug, other_team_slug):
    old_team = get_object_or_404(Team, lp_name=team_slug)
    new_team = get_object_or_404(Team, lp_name=other_team_slug)
    if old_team == 'ubuntu-lococouncil' or new_team == 'ubuntu-lococouncil':
        request.user.message_set.create(message=ugettext('You are not allowed to merge this team with another one.'))
        return redirect('/')
    #check if user has needed rights
    is_admin = launchpad.is_admin_or_owner(request.user.username, old_team) and \
               launchpad.is_admin_or_owner(request.user.username, new_team)
    if not is_admin:
        request.user.message_set.create(message=ugettext('You are not allowed to merge this team with another one.'))
        return redirect(old_team)

    msgs = merge_teams(old_team, new_team)
    for msg in msgs:
        request.user.message_set.create(message=msg)

    return render_to_response('teams/merge_teams.html',
                              {'team_object': old_team,
                               'other_team_object': new_team,
                               'user': request.user,
                              }, RequestContext(request))


@login_required
def team_edit(request, team_slug):
    #get team
    if team_slug == 'ubuntu-lococouncil':
        raise Http404
    team_object = get_object_or_404(Team, lp_name=team_slug)

    #check if user has needed rights
    is_on_lc = launchpad.is_user_on_loco_council(request.user)
    is_contact = launchpad.is_admin_or_owner(request.user.username, team_object)
    if not is_on_lc and not is_contact:
        request.user.message_set.create(message=ugettext('You are not allowed to make changes to the team details.'))
        return redirect(team_object)

    if request.method == "POST":
        if is_on_lc:
            form = forms.LoCoCouncilForm(data=request.POST, instance=team_object)
        else:
            form = forms.UpdateTeamForm(data=request.POST, instance=team_object)
        if form.is_valid():
            form.save()
            request.user.message_set.create(message=ugettext('Thanks, your team\'s details have been saved.'))
            return redirect(team_object)
        else:
            request.user.message_set.create(message=ugettext('Recent changes to your team\'s details could not be saved.'))
    else:
        if is_on_lc:
            form = forms.LoCoCouncilForm(instance=team_object)
        else:
            form = forms.UpdateTeamForm(instance=team_object)

    return render_to_response('teams/team_update.html',
                    {'form': form,
                     'team': team_object,
                    }, RequestContext(request))


def xml_team_data(request):
    XMLSerializer = serializers.get_serializer("xml")
    xml_serializer = XMLSerializer()
    xml_serializer.serialize(Team.objects.all())
    data = xml_serializer.getvalue()
    response = http.HttpResponse(data, mimetype='text/xml')
    response['Content-Disposition'] = 'filename=locoteams.xml'
    return response
