# -*- coding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth import models as auth_models
from django.db.models.signals import post_save
from common.launchpad import (get_user_timezone, lp_login, get_mugshot_url, set_user_openid)

import pytz
import logging
import sys
st_handler = logging.StreamHandler(sys.stderr)
formatter = logging.Formatter("[%(asctime)s] - %(levelname)s - %(module)s - %(message)s")
st_handler.setFormatter(formatter)

log = logging.getLogger(__name__)
log.addHandler(st_handler)
log.setLevel(logging.DEBUG)


class UserProfile(models.Model):
    " Store profile information about a user "

    user = models.OneToOneField(auth_models.User)
    realname = models.CharField(_("Real Name"), max_length=150, blank=True)
    tz = models.CharField(verbose_name=_('Timezone'), max_length=32, default='UTC')
    mugshot = models.URLField(verbose_name=_("Mugshot"), max_length=150, blank=True, verify_exists=False, null=True)

    blog = models.URLField(verbose_name=_('Blog URL'), blank=True, null=True, verify_exists=False)
    twitter = models.CharField(verbose_name=_('Twitter ID'), max_length=32, blank=True, null=True)
    identica = models.CharField(verbose_name=_('Identi.ca ID'), max_length=32, blank=True, null=True)
    picasa = models.CharField(verbose_name=_('Picasa ID'), max_length=32, blank=True, null=True)
    flickr = models.CharField(verbose_name=_('Flickr ID'), max_length=32, blank=True, null=True)
    facebook = models.CharField(verbose_name=_('Facebook ID'), max_length=32, blank=True, null=True)
    irc = models.CharField(verbose_name=_('IRC Nick'), max_length=32, blank=True, null=True)
    aim = models.CharField(verbose_name=_('AOL IM Nick'), max_length=32, blank=True, null=True)
    xmpp = models.CharField(verbose_name=_('XMPP IM Nick'), max_length=32, blank=True, null=True)

    class Meta:
        ordering = ('user__username',)

    def __unicode__(self):
        try:
            if self.realname:
                return "%s (%s)" % (self.user.username, self.realname)
            return "%s" % self.user.username
        except:
            return "Unknown Profile"

    def get_timezone(self):
        try:
            return pytz.timezone(self.tz)
        except:
            return pytz.utc
    timezone = property(get_timezone)

    def tolocaltime(self, dt):
        as_utc = pytz.utc.localize(dt)
        return as_utc.astimezone(self.timezone)

    def fromlocaltime(self, dt):
        local = self.timezone.localize(dt)
        return local.astimezone(pytz.utc)


def _getUserProfile(self):
    if not self.is_authenticated():
        return UserProfile()

    profile, created = UserProfile.objects.get_or_create(user=self)

    if created:
        profile.tz = get_user_timezone(self.username)
        profile.save()

    return profile


def _getAnonProfile(self):
    return UserProfile()

auth_models.User.profile = property(_getUserProfile)
auth_models.AnonymousUser.profile = property(_getAnonProfile)


def create_userprofile(sender, instance, lp=None, USER_BLACKLIST=(u"root", u"admin"), need_update=False, **kwargs):
    profile, created = UserProfile.objects.get_or_create(user=instance)
    if instance.username in USER_BLACKLIST:
        log.error('%s tried to create a userprofile, but exists in USER_BLACKLIST' % instance)
        return profile

    if need_update or created:
        if not lp:
            lp = lp_login()
        if lp:
            try:
                log.debug("Updating %s" % instance)
                lp_user = lp.people[instance.username]
                profile.tz = lp_user.time_zone or "UTC"
                profile.realname = lp_user.display_name or instance.username
                profile.mugshot = get_mugshot_url(lp_user.mugshot_link)
                profile.save()
            except Exception:
                log.error("Error updating %s" % instance)
    return profile

post_save.connect(create_userprofile, sender=auth_models.User, dispatch_uid=__name__)


def create_profile(username):
    user, created = auth_models.User.objects.get_or_create(username=username)
    set_user_openid(user, force=True)
    return user.get_profile()
