# -*- coding: utf-8 -*-
from django.db import models
#pylint: disable-msg=E1102
from django.utils.translation import ugettext_lazy as _

from teams.models import Country, Continent
from common.mixins import LocalTimeMixin

import pytz
import datetime


class VenueManager(models.Manager):
    """
    a manager class for Venues
    """
    def next_5_events(self):
        """ a list with the next 5 events """
        return self.filter(date_end__gt=datetime.datetime.now()).order_by('date_end')[:5]


class Venue(models.Model, LocalTimeMixin):
    """
    a venue
    """
    name = models.CharField(help_text=_('Name of the Venue'), max_length=150)
    continent = models.ForeignKey(Continent, verbose_name=_('Continent'), null=True)
    country = models.ForeignKey(Country, verbose_name=_('Country'), null=True)
    spr = models.CharField(_("State/Province/Region"), max_length=50, null=True, blank=True)
    city = models.CharField(help_text=_('City Name'), max_length=150, null=True, blank=True)
    address = models.CharField(help_text=_('Address with Street and Number'), max_length=150, null=True, blank=True)
    longitude = models.FloatField(help_text=_('Longitude in Degrees East'), null=True, blank=True)
    latitude = models.FloatField(help_text=_('Latitude in Degrees North'), null=True, blank=True)
    tz = models.CharField(max_length=32, verbose_name=_('Timezone'), default='UTC', choices=[(tz, tz) for tz in pytz.all_timezones], blank=False, null=False)
    venue_url = models.URLField(help_text=_('URL for the Venue Homepage'), verbose_name=_('URL of the Venue'),
                                max_length=200, verify_exists=False, blank=True, null=True)
    comment = models.TextField(help_text=_('Comment about the Venue'), blank=True, null=True)

    objects = VenueManager()

    class Meta:
        unique_together = (('name', 'country', 'city'), ('longitude', 'latitude'), )

    def __unicode__(self):

        if self.spr and self.city and self.country:
            return u'%s (%s, %s, %s)' % (self.name, self.city, self.spr, self.country)
        if self.city and self.country:
            return u'%s (%s, %s)' % (self.name, self.city, self.country)
        else:
            return u'%s' % self.name

    @models.permalink
    def get_absolute_url(self):
        """ get the absolute url for the venue """
        #suppress Instance of 'ForeignKey' has no 'slug' member
        #pylint: disable-msg=E1101
        if self.country:
            country = self.country.slug
        else:
            country = u'no_country'
        return ('venue-detail', [country, self.id])

    def save(self, *args, **kargs):
        if self.id:
            from events.models import TeamEvent
            try:
                old = Venue.objects.get(id=self.id)
            except Venue.DoesNotExist:
                # Object is new, but has a fixed id'
                pass
            else:
                if old.tz != self.tz:
                    events = TeamEvent.objects.filter(venue=self)
                    for e in events:
                        old_offset = old.tolocaltime(e.date_begin).utcoffset()
                        new_offset = self.tolocaltime(e.date_begin).utcoffset()
                        e.date_begin = e.date_begin + (old_offset - new_offset)
                        e.date_end = e.date_end + (old_offset - new_offset)
                        e.save()
        return super(Venue, self).save(*args, **kargs)


def venues_without_country():
    return Venue.objects.filter(country__isnull=True)
